/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name:  $
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.ext.java5.authentication.jmx;

import java.io.IOException;
import java.net.MalformedURLException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.remote.JMXConnectorServer;
import javax.management.remote.JMXConnectorServerFactory;
import javax.management.remote.JMXServiceURL;
import javax.management.remote.MBeanServerForwarder;
import javax.management.remote.rmi.RMIConnectorServer;
import javax.security.auth.login.Configuration;

import net.sf.jguard.core.authorization.policy.AccessControllerUtils;
import net.sf.jguard.ext.SecurityConstants;

/**
 * utility class used to securize JMX.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 */
public class JMXHelper {
	
	private static final String COM_SUN_JNDI_RMI_REGISTRY_REGISTRY_CONTEXT_FACTORY = "com.sun.jndi.rmi.registry.RegistryContextFactory";
	private static final Logger logger = Logger.getLogger(JMXHelper.class.getName());
	private static final String JAVA_NAMING_FACTORY_INITIAL = "java.naming.factory.initial";
	
	public static void enableJMXSecurity(String applicationName,Map opts,Configuration configuration,AccessControllerUtils authorizationUtils) {
		// JMX MBeanServer creation
        String mbeanServerForConnector = (String)opts.get(SecurityConstants.MBEAN_SERVER_FOR_CONNECTOR);
        
        	MBeanServer mbs = null;
        	if(mbeanServerForConnector==null ||mbeanServerForConnector==""||"new".equals(mbeanServerForConnector)){
        		mbs = MBeanServerFactory.createMBeanServer(applicationName);
        		logger.info("Starting JMX Server ...");
        	}else if(mbeanServerForConnector.startsWith("position#")){
        		//user provides position#$position 
        		//for example 'position#2'
        		mbeanServerForConnector = mbeanServerForConnector.substring(0, 9);
        		int position = Integer.parseInt(mbeanServerForConnector);
	        	//does each webapp create its own MBeanServer
	        	//or do webapps share their MBeans on the same MBeanServer ?
        		ArrayList mbeanServers = MBeanServerFactory.findMBeanServer(null);
	        	mbs = (MBeanServer)mbeanServers.get(position);
        	
        	}else{
        		//user provides serverName#$position    
        		//for example 'myPrettyServer#3'
        		//if user provides 'myPrettyServer#' or 'myPrettyServer',
        		//we will get the first MBeanServer => like 'myPrettyServer#0'
        		List tokens = Arrays.asList(mbeanServerForConnector.split("#"));
        		String mbeanServerName = (String)tokens.get(0);
        		String position = null;
        		if(tokens.size()>=2){
        			position = (String)tokens.get(1);
        		}else{
        			position ="0";
        		}
        		ArrayList mbeanServers = MBeanServerFactory.findMBeanServer(mbeanServerName);
        		
        		mbs = (MBeanServer)mbeanServers.get(Integer.parseInt(position));
        		
        	}
        	String rmiRegistryHost=SecurityConstants.DEFAULT_RMI_REGISTRY_HOST;
        	if(opts.get(SecurityConstants.RMI_REGISTRY_HOST)!=null){
        		rmiRegistryHost=(String)opts.get(SecurityConstants.RMI_REGISTRY_HOST);
        	}

        	int rmiRegistryPort=SecurityConstants.DEFAULT_RMI_REGISTRY_PORT;
        	if(opts.get(SecurityConstants.RMI_REGISTRY_PORT)!=null){
        		try{
        		rmiRegistryPort=Integer.parseInt((String)opts.get(SecurityConstants.RMI_REGISTRY_PORT));
        		}catch(NumberFormatException e){
        			logger.warning("RmiRegistry port in web.xml is not a number. Using default.");
        		}
        	}

        	try{
        		// TODO handle custom rmiRegistryHost
    			LocateRegistry.createRegistry(rmiRegistryPort);
//        		LocateRegistry.getRegistry(rmiRegistryHost,rmiRegistryPort);

        	}catch(RemoteException e){
        		logger.severe("Could not found rmiRegistry : "+e.getMessage());
        	}

        	String serviceURL = "service:jmx:rmi://localhost/jndi/rmi://"+rmiRegistryHost+":"+rmiRegistryPort+"/"+applicationName;
    		JMXServiceURL url=null;
    		try {
    			url=new JMXServiceURL(serviceURL);
    		} catch (MalformedURLException e) {
    			logger.severe("MalformedURLException : "+e);
    		}
    		logger.info("JMX Server URL : "+url.toString());
    		
    		JGuardJMXAuthenticator jmxAuthenticator= null;
    		
    		//we use the COnfiguration from the JVM
    		if(configuration==null){
    			jmxAuthenticator=new JGuardJMXAuthenticator(applicationName,Thread.currentThread().getContextClassLoader());
    		}else{
    			jmxAuthenticator=new JGuardJMXAuthenticator(applicationName,Thread.currentThread().getContextClassLoader(),configuration);
    		}

    		Map options=new HashMap();
    		options.put(JMXConnectorServer.AUTHENTICATOR,jmxAuthenticator);
    		options.put(JAVA_NAMING_FACTORY_INITIAL,JMXHelper.COM_SUN_JNDI_RMI_REGISTRY_REGISTRY_CONTEXT_FACTORY);
    		options.put(RMIConnectorServer.JNDI_REBIND_ATTRIBUTE,"true");
    		try {
    			JMXConnectorServer connectorServer=	JMXConnectorServerFactory.newJMXConnectorServer(url,options,mbs);
    			
    			//we are in 'local' mode
    			if(authorizationUtils.getAccessController()!=null){
//    				MBeanServerForwarder msf = MBSFInvocationHandler.newProxyInstance(authorizationUtils.getAccessController());
    				MBeanServerForwarder msf = new MBeanServerGuard(authorizationUtils.getAccessController());
    				connectorServer.setMBeanServerForwarder(msf);
    			}
    			connectorServer.start();
    		} catch (IOException e) {
    			logger.severe("IOException : "+e);
    		}

	}

}
